#! /bin/bash
# -----------------------------------------------------------------------------
# f2cad: Convert Fortran to C++ AD Types: Copyright (C) 2005-10 Bradley M. Bell
# Authors:  Bradley M. Bell    bradbell at washington dot edu
# License:  GNU Public License Version 2 or higher
# -----------------------------------------------------------------------------
# $begin add2lib.sh$$ $newlinech #$$
# $spell
#	cp
#	cpp
#	mysub
# $$
#
# $section Adding a Fortran Routine to the f2cad Libraries$$
#
# $head Required Software$$
# $href%http://netlib.sandia.gov/f2c/index.html%f2c%$$,
# $href%http://www.gnu.org/software/autoconf/%autoconf%$$,
# $href%http://www.gnu.org/software/automake/%automake%$$,
# $href%http://www.seanet.com/~bradbell/omhelp/installunix.xml%omhelp%$$,
# $href%http://http://www.coin-or.org/CppAD/%cppad%$$,
# $href%http://www.fadbad.com/fadbad.html%fadbad%$$,
# $href%https://projects.coin-or.org/ADOL-C%adolc%$$
# 
# $head Example Steps$$
# $list number$$
# Copy the file Fortran source code for the routine you are adding to
# the $code fortran$$ subdirectory of the distribution directory.
# For the mysub example, 
# execute the following command in the distribution directory:
# $codei%
# 	cp mysub.f fortran/mysub.f
# %$$
#
# $lnext
# In the f2cad distribution directory, execute the command
# $codei%
#	./add2lib.sh %name%
# %$$
# where $icode name$$ is the name of the Fortran routine.
# For the mysub example, execute the following command 
# $codei%
#	./add2lib.sh mysub
# %$$
#
# $lnext
# In the f2cad distribution directory, execute the command
# $codei%
#	./add2lib.sh use_new_files
# %$$
# This command will copy the $icode%*%.new%$$ files to the corresponding
# files (but first it will back up the corresponding files 
# in $icode%*%.old%$$).
# If $icode name$$ is not $code mysub$$, the initial version of 
# $codei%
#	example/%name%.cpp
# %$$
# will return $code test_none$$.
# You can edit this file to create a real test and example of $icode name$$.
#
# $lnext
# In the file $code build.sh$$, make sure that
# $code F2C_DIR$$, $code ADOLC_DIR$$, and $code FADBAD_DIR$$
# have the proper values for your system.
# Then execute the command
# $codep
#	./build.sh all
# %$$
# 
# $lnext
# You can undo the effect of the $code add2lib.sh$$ 
# commands above by executing
# $codep
#	./add2lib.sh use_old_files
# $$
#
# $lend
#
# $end
if echo "$1" | grep '\.'
then
	echo "add2lib.sh there should be no periods in the routine_name"
	exit 1
fi
if [ "$1" = "" ] 
then
	echo "usage: add2lib (routine_name | use_new_files | use_old_files)"
	echo "where the Fortran file fortran/routine_name.f exists"
	exit 1
fi
if [ "$1" = "use_new_files" ]
then
	#
	for P in adolc cppad fadbad
	do
		echo "mv example/$P/makefile.am example/$P/makefile.old"
		echo "mv example/$P/makfile.new example/$P/makefile.am"
		mv example/$P/makefile.am example/$P/makefile.old
		mv example/$P/makefile.new example/$P/makefile.am
	done
	#
	echo "mv example/run.cpp example/run.old"
	echo "mv example/run.new example/run.cpp"
	mv example/run.cpp example/run.old
	mv example/run.new example/run.cpp
	#
	echo "mv omh/library.omh omh/library.old"
	echo "mv omh/library.new omh/library.omh"
	mv omh/library.omh omh/library.old
	mv omh/library.new omh/library.omh
	#
	for file in \
		lib/adolc/makefile \
		lib/cppad/makefile \
		lib/fadbad/makefile
	do
		echo "mv $file.am  $file.old"
		echo "mv $file.new $file.am"
		mv $file.am  $file.old
		mv $file.new $file.am
	done
	echo "Execute ./build_sources.sh to create new versions of the directories"
	echo "cpp, f2c.p, f.omh, p.omh, f2cad."
	exit 0
fi
if [ "$1" = "use_old_files" ]
then
	name=`diff example/cppad/makefile.am example/cppad/makefile.old | \
		sed -n -e '/^</p'`
	name=`echo $name | sed -e 's|< *\.\./||' -e 's|\.cpp.*||'`
	echo "rm omh/$name.f.omh"
	rm omh/$name.f.omh
	#
	for P in adolc cppad fadbad
	do
		echo "mv example/$P/makefile.old example/$P/makefile.am"
		mv example/$P/makefile.old example/$P/makefile.am
	done
	#
	echo "mv example/run.old example/run.cpp"
	mv example/run.old example/run.cpp
	#
	echo "mv omh/library.old omh/library.omh"
	mv omh/library.old omh/library.omh
	#
	for file in \
		lib/adolc/makefile \
		lib/cppad/makefile \
		lib/fadbad/makefile
	do
		echo "mv $file.old $file.am"
		mv $file.old $file.am
	done
	#
	echo "Remove new new file from the fortran directory and then execute"
	echo "	./build_sources.sh"
	echo "to restore cpp, f2c.p, f.omh, p.omh, and f2cad to old values."
	echo
	echo "Not deleting example/$name.cpp (you may have edited this file)."
	exit 0
fi
name="$1"
if [ ! -e fortran/$name.f ]
then
	echo "add2lib.sh: cannot find fortran/$name.f"
	exit 1
fi
# -------------------------------------------------------------------------- 
for P in adolc cppad fadbad
do
	echo "Creating example/$P/makefile.new"
	sed < example/$P/makefile.am > example/$P/makefile.new \
		-e "s|SOURCES *=.*|&\\n\t../$name.cpp \\\\|"
done
# --------------------------------------------------------------------------
echo "Creating example/run.new"
externText="// ../add2lib.sh uses this comment for extern command"
runText="// ../add2lib.sh uses this comment for run command"
sed < example/run.cpp > example/run.new \
	-e "s|$externText *|$externText \nextern test_result $name(void);|" \
	-e "s|$runText *|$runText \n\tok \&= run( $name , \"$name\" );|"
# --------------------------------------------------------------------------
echo "Creating omh/library.new"
list=`
	sed -n < omh/library.omh -e '/$contents/,/\$\$/p' |
	sed -e '/$contents/d' -e '/\$\$/d' -e 's|example/||' -e 's|[%\t]||g' |
	sort
`
newline="
"
list=`echo "$list$newline$name.cpp" | sort`
ordered=""
for tmp in $list
do
	ordered="$ordered%$tmp"
done
sed -n < omh/library.omh -e '1,/^$contents/p'           > omh/library.new
echo $ordered | \
	sed -e 's|^%|\texample/|' -e 's|%|%\n\texample/|g' >> omh/library.new
sed -n < omh/library.omh -e '/^%\$\$/,$p'               >> omh/library.new
# --------------------------------------------------------------------------
for P in adolc cppad fadbad
do
echo "Creating lib/$P/makefile.new"
sed < lib/$P/makefile.am > lib/$P/makefile.new \
-e "s|libf2c_${P}_a_SOURCES *= *|& \\\\\n\t\$(top_builddir)/cpp/$name.cpp |"  
done
# --------------------------------------------------------------------------
# Trick to avoid OMhelp from processing the code below:
dollar="\$"
#
if [ "$name" == "mysub" ]
then
echo "$name == mysub"
exit 0
# ----------------------------------------------------------------------------
echo "Creating example/mysub.cpp."
cat << EOF > example/mysub.cpp
/* ----------------------------------------------------------------------------
  f2cad: Convert Fortran to C++ AD Types: Copyright (C) 2005-10 Bradley M. Bell
  Authors:  Bradley M. Bell    bradbell at washington dot edu
  License:  GNU Public License Version 2 or higher
-----------------------------------------------------------------------------*/
/*
${dollar}begin mysub\$\$ 
\$comment  The omhelp commands are documented in
	http://www.seanet.com/~bradbell/omhelp/
\$\$
\$spell
	cpp
	hpp
	mysub
	myfun
	bool
	doublereal
	adolc
	cppad
	fadbad
\$\$

\$section mysub\$\$

\$index mysub\$\$

\$head Prototype\$\$
\$include p.omh/mysub.p.omh\$\$

\$children%
	f.omh/mysub.f.omh
%\$\$
\$head Fortran Source\$\$
\$cref%mysub.f%\$\$


\$head Description\$\$
This is an example and test of the routine mysub.

\$codep */
# include <f2cad/mysub.hpp>

test_result mysub(void)
{	bool ok = true;

	// independent variables 
	integer     n = 1;        // number of independent variables
	doublereal  x[1];         // vector of independent variables
	x[0]          = 3.;       // value  of independent variables
	f2cad::Independent(n, x); // declare x as independent variable vector

	// dependent variables 
	integer     m = 1;        // number of dependent variables
	doublereal  f[1];         // vector of dependent variables
	f2cad::mysub_(x, f);      // compute value f[0] = x[0] * x[0]
	f2cad::Dependent(m, f);   // declare f as dependent variable vector

	// Compute derivative of f w.r.t x using f2cad common interface.
	// This general interface works with adolc, cppad, and fadbad.
	// See GetStarted for package specific more efficient code.
	double p;
	p   = f2cad::Partial<doublereal>(0, 0);
	ok &= f2cad::near_equal(p, 2*f2cad::Value(x[0]), 1e-10, 1e-10);

	if( ok )
		return test_pass;
	return test_fail;
}
/* \$\$
\$end
*/
EOF
else
# ----------------------------------------------------------------------------
echo "Creating an empty version of example/$name.cpp."
echo "You can edit this file to make it a better example / test."
cat << EOF > example/$name.cpp
/* ----------------------------------------------------------------------------
  f2cad: Convert Fortran to C++ AD Types: Copyright (C) 2005-10 Bradley M. Bell
  Authors:  Bradley M. Bell    bradbell at washington dot edu
  License:  GNU Public License Version 2 or higher
-----------------------------------------------------------------------------*/
/*
${dollar}begin $name\$\$ 
\$comment  The omhelp commands are documented in
	http://www.seanet.com/~bradbell/omhelp/
\$\$
\$spell
	$name
	cpp
	hpp
	bool
	doublereal
	adolc
	cppad
	fadbad
\$\$

\$section $name\$\$

\$index $name\$\$

\$head Prototype\$\$
\$include p.omh/$name.p.omh\$\$

\$children%
	f.omh/$name.f.omh
%\$\$
\$head Fortran Source\$\$
\$cref%$name.f%\$\$


\$head Description\$\$
Change this paragraph to a description of this example / test.


\$codep */
# include <f2cad/$name.hpp>

test_result $name(void)
{
	// Replace the following line by an example / test of $name
	// that returns test_pass if the test passes and test_fail otherwise.
	return test_none;
}
/* \$\$
\$end
*/
EOF
# ----------------------------------------------------------------------------
fi
